#include "shell.h"
#include "console.h"
#include "globals.h"

extern char gsh_message[GSH_MESSAGE_LEN];
static enum { Shrinking, Off, Growing, On } console_status = Off;
static float height = 0.1f;


// outputs text pointed by string on OpenGL screen at location x,y
void OutputString (  float x,  float y,  char *string )
{
	int length;

	length = strlen ( string );
	glRasterPos2f ( x, y );
	
	for (int i = 0; i < length; i++ )
		glutBitmapCharacter ( GLUT_BITMAP_9_BY_15, string[i] );
}

// Saves whatever is currently in the front frame buffer to the file specified
// Returns -1 on error, 0 on success;
int save_screenshot(char *fname)
{
	int err;
	unsigned char *image;

	image = (unsigned char *) malloc(WIDTH * HEIGHT * 4); // RGBA

	glPixelStorei(GL_PACK_ALIGNMENT, 1); // set correct alignment
	glReadBuffer(GL_FRONT); // we want to read the back buffer

	// read the entire display window
	glReadPixels(0, 0, WIDTH, HEIGHT, GL_RGBA, GL_UNSIGNED_BYTE, image);

	// Now store it to a file
	err = save_BMP(fname, image, WIDTH, HEIGHT);

	free(image); // we do not want a memory leak!

	return err;
}


// glut display function. Called each time screen is to be redrawed. Can be forced
// by calling glutPostRedisplay().
// This is where all the drawing related stuff takes place.

void display()
{
	// Everything we do here gets done in the back buffer. The buffers are swaped
	// after we are done with the changes.

	// Clear the frame-buffer
	glClear(GL_COLOR_BUFFER_BIT );
	glPushMatrix();
	glTranslatef(x_pos,y_pos,0);
	//if(enableTexture) glEnable(GL_TEXTURE_2D);	// May not be used in Linux
	myMesh.display(); // display  the mesh
	//if(enableTexture) glDisable(GL_TEXTURE_2D); // May not be used in Linux
	//OutputString(x_pos, y_pos,message);
	glPopMatrix();
	glFlush();	// refresh the buffer

	gcon_Render(height); // draw our cool shell interface
	// Swap the frame buffer. Make the back buffer the new front buffer and
	// vice-versa. To avoid flicker we use two buffers. While we display one,
	// we make changes to the second hidden buffer.
	
	glutSwapBuffers();
}


// ths animates our shell
static void redisplay(int delay)
{
    if (console_status == Growing) {
        gcon_SetBackgroundColor(0.0f, (1.0-height/MAXHEIGHT)*0.3f, (height/MAXHEIGHT)*0.5f, BACKGROUND_ALPHA);
        height += INCREMENT;
    }
    else if (console_status == Shrinking) {
        gcon_SetBackgroundColor(0.0f, (1.0-height/MAXHEIGHT)*0.3f, (height/MAXHEIGHT)*0.5f, BACKGROUND_ALPHA);
        height -= INCREMENT;
    }
    if (height > MAXHEIGHT) {
        height = MAXHEIGHT;
        console_status = On;
    }
    else if (height < 0.1f) {
        height = 0.1f;
		//gcon_SetBackgroundColor(0.0f, 0.5f, 0.0f, BACKGROUND_ALPHA);
        console_status = Off;
    }
    else {
        glutTimerFunc(delay, redisplay, delay);
    }
    glutPostRedisplay();
}

// glut reshape function. Called when the window is resized. Also called when the window
// is created. Initialize the Projection matrix here.
void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);

	// Set the projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(-1.0, 1.0, -1.0, 1.2);

	// Initialize the modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


// glut keyboard function.
void keyboard(unsigned char key, int x, int y)
{

	if (key == '`') {
        if (console_status == Off || console_status == Shrinking) {
            console_status = Growing;
        }
        else {
            console_status = Shrinking;
        }
        redisplay(50);
        return;
    }

	if (console_status == Shrinking || console_status == Off) {
        gcon_ExecKeyBind(key, GLUT_DOWN);
	}
    else {
        gcon_Putch(key);
    }
	// We might have changed something. Force a re-display
	glutPostRedisplay();
}

static void keyboardup(unsigned char key, int x, int y)
{
    if (console_status == Shrinking || console_status == Off) {
        gcon_ExecKeyBind(key, GLUT_UP);
    }
}

// glut special key handler
void special(int key, int x, int y)
{
	// As a demonstration, we change two global variables based on
	// the key press. These variables are used to translate the text
	// we have drawn.
	if (console_status == Shrinking || console_status == Off) {
        gcon_ExecSpecialBind(key, GLUT_DOWN);
	switch(key)	{
		case GLUT_KEY_UP:
			y_pos+=increment;
			break;

		case GLUT_KEY_DOWN:
			y_pos-=increment;
			break;
		case GLUT_KEY_LEFT:
			x_pos-=increment;
			break;
		case GLUT_KEY_RIGHT:
			x_pos+=increment;
			break;
		default:
			break;
	}
	
    }
    else {
        gcon_SpecialKey(key);
    }
	// We might have changed something. Force a re-display
	glutPostRedisplay();
}

static void specialup(int key, int x, int y)
{
    if (console_status == Shrinking || console_status == Off) {
        gcon_ExecSpecialBind(key, GLUT_UP);
    }
}

// glut idle function. Called at every 'clock tick'
void idle(void)
{
	// Put anything that needs to be updated all the time here.
	// Any sort of animation would require you to use this fundtion.
	// Remember to call glutPostRedisplay() once you've made the changes
	// to force the screen to update

	// We might have changed something. Force a re-display
	glutPostRedisplay();
}

// Initialize all opengl related stuff
void gl_init()
{
	glClearColor(0.0, 0.0, 0.0, 0.0); // Set clear color
}

// Initialize all glut related stuff, including callback functions.
void glut_init(int *argc, char **argv)
{
	// Call glut's initilization
	glutInit(argc, argv);

	// double buffer, RGB color model, depth buffer
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB); // | GLUT_DEPTH);

	// Now create a window of the required size
	glutInitWindowSize(WIDTH, HEIGHT);
	glutInitWindowPosition(0, 0);
	glutCreateWindow(TITLE);

	// glut Callback functions
	glutDisplayFunc(display); // display
	glutKeyboardFunc(keyboard); // keyboard
	glutSpecialFunc(special); // special keys
	glutReshapeFunc(reshape); // reshape
	glutIdleFunc(idle); // idle


	// These are callbacks for the mouse. Implement yourself if needed
	glutMouseFunc(NULL);
	glutMotionFunc(NULL);
}

int LoadTexture(char *fname)
{
  // Try loading a BMP image file - for illustration
	int w, h;
	unsigned char *img;
	
	int success = load_padded_BMP(fname, &img, &w, &h);
	if(success!=0)
	{
		sprintf(gsh_message,"could not load bmp file %s",fname);
		//enableTexture = false;
		return 0;
	}
		
	sprintf(gsh_message,"Loaded BMP file. Width = %d, Height = %d\n", w, h);
	
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glClear(GL_COLOR_BUFFER_BIT);

	// generate 1 texture
	glGenTextures(1, &texName);


	// initialize the texture
	glBindTexture(GL_TEXTURE_2D, texName);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, img);

 
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glEnable(GL_TEXTURE_2D); // default texture enable
	return 1;
}

void init()
{
	myMesh.Initialize(meshW,meshH);	// build the mesh
	LoadTexture("image.bmp");	
	// now we make a test projective transform
	Mat3 S;
	S.makeScale(0.5,0.75);
	H.ident();
	H.mpy(H,S);
  	S.makeRot(45.0*M_PI/180.0);
  	H.mpy(H,S);
	H.prnt();
	sprintf(message,"Welcome to IBMR. Please see console for options");
	cout << "List of keys:" << endl
         << "t : Toggle Texture ON/OFF\n"
		 << "m : Toggle display mode between VERTEX/LINE/FILL\n"
		 << "h : Apply Transformation recursively\n"
		 << "r : Reset mesh to start\n"
		 << "q,Esc : Quit\nArrow keys: change mesh location\n"
		 << "` : Grow/Srink the shell\n";
}

int main(int argc, char** argv)
 {
	// First initialize glut
	glut_init(&argc, argv);
	// Now initialize OpenGL stuff
	gl_init();

	init_gcon();
	// Now initialize program specific stuff
	init();
	// Finally enter the glut infinite loop 
	glutMainLoop();

	return 0;
}
